#**************************************************************************
#    copyright            : (C) 2005 by Michael Zimmermann
#    email                : webmaster@incunabulum.de
# ***************************************************************************/

#***************************************************************************
#                                                                         *
#    This program is free software; you can redistribute it and/or modify  *
#    it under the terms of version 2 of the GNU General Public License as  *
#    published by the Free Software Foundation;                            *
#                                                                          *
# **************************************************************************/
  
import logging
import sys

from qt import *
from ZODB import DB, FileStorage
from mainwindowbase import MainWindowBase
from pathconfig import PathConfig
from objectvisualization import ObjectVisualization

class MainWindow(MainWindowBase):
    """
    MainWindow . This class is derived from
    the pyuic-ed MainWindowBase class
    """
    
    def __init__(self):
        """
        Default constructor
        """
        MainWindowBase.__init__(self)
        self.__logger = logging.getLogger('MainWindow')        
        self.__logger.debug('MainWindow constructor called')
        self.__connected = False
        self.__path = ''
        self.__pythonPath = sys.path
        self.__stack = []
        
    def __rebuildStack(self, lvitem):
        """
        Rebuilds the stack that stores lvitem (and all parent items  up
        to but not including the root element as QListViewItems
        
        lvitem is selected item
        """
        self.__logger.debug('Rebuilding stack from lvitem with id %s' % str(lvitem.text(0)))
        self.__stack = []
        self.__stack.append(lvitem)
        
        while lvitem.parent():
            lvitem = lvitem.parent()
            self.__stack.insert(0, lvitem)
        # self.__stack.insert(0, lvitem)
        self.__stack.pop(0)
            
    def __getObject(self, lvitem):
        """
        Returns the object from the zobdd that is denoted by lvitem.
        """
        self.__logger.debug('Get zodb object for lvitem wiwth id %s' % str(lvitem.text(0)))
        self.__rebuildStack(lvitem)
        
        obj = self.__root
        for sub in self.__stack:
            qlvitem = sub
            obj = obj[str(qlvitem.text(0))]
        return obj
    
    def __showObjectInfo(self, id, obj):
        """
        Presents information of object obj in the right info pane
        """
        try:
            import obj.__class__.__module__
        except:
            self.__logger.debug('Could not import module %s' % obj.__class__.__module__)
        self.objectInfo.setText('')
        viz = ObjectVisualization()
        txt = viz.getVisualization(id, obj)
        self.objectInfo.append(txt)
        self.objectInfo.setCursorPosition(0,0)

        
    def  connectDb(self):
        self.__logger.debug('Connecting to db')
        self.disconnectDb()
        path = str(self.path.text())
        if not path:
            QMessageBox.warning(self, "No zodb given",
                                "Please select a zodb to open", 0,0,1)
            return
        try: 
            self.__storage = FileStorage.FileStorage(path)
            self.__db = DB(self.__storage)
            self.__connection = self.__db.open()
            self.__root = self.__connection.root()
            self.__connected = True
            self.__stack.append(QListViewItem(self.tree, 'Root', str(self.__root.__class__)))
            self.__stack[0].setExpandable(True)
        except: 
            QMessageBox.warning(self, "Connection failed",
                                "Could not connect to database. Wrong file?", 0,0,1)
    
    def disconnectDb(self):
        """
        Slot disconnects from the db
        """
        if self.__connected:
            self.__logger.debug('Closing db connection')
            get_transaction().abort()
            self.__connection.close()
            self.__db.close()
            self.__storage.close()
            
        self.__connected = False
        self.__stack = []
        self.tree.clear()
        self.objectInfo.clear()
        
    def getPath(self):
        """
        Slot used to select zodb file
        """
        self.__logger.debug('Get path to zodb')
        path = QFileDialog.getOpenFileName("", "", self, "FileDialog",
                                           "Select path to zodb")
        if path:
            self.__path = str(path)
            self.path.setText(self.__path)
    
    def configurePath(self):
        """
        Slot used to configure additional custom python path
        """
        self.__logger.debug('Configure custom python path')
        dlg = PathConfig(self.__pythonPath)
        dlg.exec_loop()
        self.__pythonPath = dlg.getPath()        
        sys.path = self.__pythonPath
        
    def closeEvent(self, event):
        """
        CloseEvent handling
        """
        self.__logger.debug('Close event, terminating application')
        self.disconnectDb()    
        event.accept()
                
    def objectExpanded(self, lvitem):
        """
        Slot called if an object is expanded
        """
        self.__logger.debug('Expanding object with id %s' % str(lvitem.text(0)))
        obj = self.__getObject(lvitem)
        
        # create list of existing ids below obj
        subIds = []
        sub = lvitem.firstChild()
        while sub:
            subIds.append(str(sub.text(0)))
            sub = sub.nextSibling()
        
        # add subobjects
        keys = list(obj.keys())
        for key in keys:
            if not subIds.count(key):
                sub = obj[key]
                lvItem = QListViewItem(lvitem, key, str(sub.__class__))
                if hasattr(sub, 'keys'):
                    lvItem.setExpandable(True)
            
    def objectSelected(self, lvitem):
        """
        Slot called if an object is selected. 
        """
        if lvitem:
            self.__logger.debug('Selected object %s' % str(lvitem.text(0)))
            obj = self.__getObject(lvitem)
            self.__showObjectInfo(str(lvitem.text(0)), obj) 
 
 
 
